%% Convert images to grayscale
ref_img = 0.2989*ref_img_clr(:,:,1)+0.5870*ref_img_clr(:,:,2)+0.1140*ref_img_clr(:,:,3); 
% plotImage(ref_img)

%% Binarize the image

% create binary image
ref_img_bw = im2bw(ref_img, D(1).binarizationLimit);
% % plotImage(ref_img_bw)

%% get blob areas
ref_img_bw = ExtractNLargestBlobs(ref_img_bw,2);
% plotImage(ref_img_bw)

ref_img_wb = ~ref_img_bw;
ref_img_wb = ExtractNLargestBlobs(ref_img_wb,1);

% plotImage(ref_img_wb)

%% create clean distinct areas
for i = 1:size(ref_img_bw,1)
    for j = 1:size(ref_img_bw,2)
        if ~ref_img_bw(i,j) && ref_img_wb(i,j)
            ref_img(i,j) = true;
        else
            ref_img(i,j) = false;
        end
    end
end

ref_img = logical(ref_img);
% plotImage(ref_img)

%% smooth edges
ref_img_smoothed = bwmorph(ref_img, 'erode', D(1).smoothingRadius);
% plotImage(ref_img_smoothed)

%% create contour lines

% fiber / water level is true at this moment

cL_1x = [];
cL_1y = [];
cL_2x = [];
cL_2y = [];
nRows = size(ref_img_bw,1);
current_cL = 1;

for i = 1:size(ref_img_bw,2)
    valueRecorded = false;
    
    if current_cL == 1
        j = nRows+1;
        while (j > 1) && ~valueRecorded
            j = j-1;
            
            if j == 1
                current_cL = 2;
            end
            
            if ~ref_img_smoothed(j,i) % changed for ref_img during debugging
                cL_1x = [cL_1x; i];
                cL_1y = [cL_1y; j];
                valueRecorded = true;
            end
        end
    end
end

for i = size(ref_img_bw,2):-1:1
    valueRecorded = false;
    
    if current_cL == 2
        j = nRows+1;
        while (j > 1) && ~valueRecorded
            j = j-1;
            
            if j == 1
                current_cL = 1;
            end
            
            if ~ref_img_smoothed(j,i) % changed for ref_img during debugging
                cL_2x = [cL_2x; i];
                cL_2y = [cL_2y; j];
                valueRecorded = true;
            end
        end
    end
end

% figure
% imshow(ref_img)
% hold on
% plot(cL_1x, cL_1y,'-r','LineWidth',2)
% plot(cL_2x, cL_2y,'-r','LineWidth',2)

%% get derivatives of contour lines
dcL_1 = diff(smooth(cL_1y,'rloess'));
dcL_2 = diff(smooth(cL_2y,'rloess'));

% figure
% plot(cL_1x(1:end-1),-dcL_1,'-b')
% hold on
% plot(cL_2x(1:end-1),-dcL_2,'-r')

%% find fiber-liquid level boundary
pos_1 = find(abs(dcL_1) > D(1).derivativeLimit);
if isempty(pos_1)
    pos_1 = length(dcL_1);
else
    pos_1 = pos_1(1);
end

pos_2 = find(abs(dcL_2) > D(1).derivativeLimit);
if isempty(pos_2)
    tempPos2 = length(dcL_2);
    pos_2 = size(ref_img_bw,2)-length(dcL_2);
else
    tempPos2 = pos_2(1);
    pos_2 = size(ref_img_bw,2)-pos_2(1);
end

P1 = [pos_1-D(1).xOffset, cL_1y(pos_1-D(1).xOffset)];
P2 = [pos_2+D(1).xOffset, cL_2y(tempPos2-D(1).xOffset)];

dy_dx_1 = abs(cL_1y(pos_1-D(1).xOffset)-cL_1y(pos_1-D(1).xOffset-D(1).dx))/D(1).dx;
dy_dx_2 = abs(cL_2y(tempPos2-D(1).xOffset)-cL_2y(tempPos2-D(1).xOffset-D(1).dx))/D(1).dx;

slope_1 = 90-atand(dy_dx_1);
slope_2 = 90-atand(dy_dx_2);

%% plotting data
radius = cL_1y(pos_1-D(1).xOffset)/2.5;

sl_1 = atand(dy_dx_1);
sl_2 = atand(dy_dx_2);

x1 = [P1(1)+radius*cosd(-sl_1), P1(1)-radius*cosd(-sl_1)];
x2 = [P2(1)+radius*cosd(sl_2), P2(1)-radius*cosd(sl_2)];

y1 = [P1(2)+radius*sind(-sl_1), P1(2)-radius*sind(-sl_1)];
y2 = [P2(2)+radius*sind(sl_2), P2(2)-radius*sind(sl_2)];

%% save the results
D(processedImage).P1 = P1;
D(processedImage).P2 = P2;
D(processedImage).slope_1 = slope_1;
D(processedImage).slope_2 = slope_2;
D(processedImage).x1 = x1;
D(processedImage).y1 = y1;
D(processedImage).x2 = x2;
D(processedImage).y2 = y2;
D(processedImage).radius = radius;

